﻿using System;
using System.ComponentModel;
using System.Windows.Media;
using IGShowcase.AirplaneSeatingChart.Resources;
using IGShowcase.AirplaneSeatingChart.Models;
using Infragistics;

namespace IGShowcase.AirplaneSeatingChart.ViewModels
{
	public sealed class SeatViewModel : INotifyPropertyChanged
	{
		#region Private members
		private static readonly Brush TransparentBrush = new SolidColorBrush(Colors.Transparent);

		private readonly Seat _seat;
		private readonly Brush _fill;
		private readonly Brush _selectedFill;
		private bool _isSelected;
		private bool _isDisabled;
		#endregion

		#region Constructors
		/// <summary>
		/// Initializes a new instance of the <see cref="SeatViewModel"/> class.
		/// </summary>
		/// <param name="id">The id.</param>
		/// <param name="seat">The seat.</param>
		/// <param name="brushes">The brushes.</param>
		public SeatViewModel(string id, Seat seat, BrushCollection brushes)
		{
			Id = id;
			_seat = seat;
			_fill = seat.IsOccupied ? brushes[0] : brushes[(int)(seat.SeatType) + 1]; ;
			_selectedFill = brushes[brushes.Count - 1];
		}
		#endregion Constructors

		#region Public Properties
		/// <summary>
		/// The ID of the map element for the map data mapping
		/// </summary>
		public string Id { get; private set; }

		/// <summary>
		/// The seat class
		/// </summary>
		public string Class { get { return _seat.Class; } }

		/// <summary>
		/// Gets the position.
		/// </summary>
		/// <value>The position.</value>
		public string Position { get { return string.Format("{0}{1}", _seat.Row, _seat.Column); } }

		/// <summary>
		/// Gets the status text.
		/// </summary>
		/// <value>The status text.</value>
		public string StatusText { get { return _seat.IsOccupied ? AppStrings.Occupied : AppStrings.Available; } }

		/// <summary>
		/// The fill brush to be used in the map. This is bound through the layer DataMapping property.
		/// </summary>
		public Brush Fill
		{
			get { return _isSelected ? _selectedFill : (_isDisabled ? TransparentBrush : _fill); }
		}

		/// <summary>
		/// Gets or sets a value indicating whether this seat is disabled when selecting seat class.
		/// </summary>
		public bool IsDisabled
		{
			get { return _isDisabled; }
			set
			{
				if (_isDisabled != value)
				{
					_isDisabled = value;
					OnPropertyChanged("IsDisabled");
					OnPropertyChanged("Fill");
				}
			}
		}

		/// <summary>
		/// Gets a value indicating whether this seat is occupied.
		/// </summary>
		public bool IsOccupied { get { return _seat.IsOccupied; } }

		/// <summary>
		/// Gets or sets a value indicating whether this seat is selected.
		/// </summary>
		public bool IsSelected
		{
			get{ return _isSelected; }
			set
			{
				if(_isSelected != value)
				{
					_isSelected = value;
					OnPropertyChanged("IsSelected");
					OnPropertyChanged("Fill");
				}
			}
		}
		#endregion Public Properties

		#region INotifyPropertyChanged Members

		public event PropertyChangedEventHandler PropertyChanged;

		private void OnPropertyChanged(string propertyName)
		{
			if (PropertyChanged != null)
			{
				PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
			}
		}

		#endregion
	}
}
