﻿//-------------------------------------------------------------------------
// <copyright file="SeatingMapViewModel.cs" company="Infragistics">
//
// Copyright (c) 2010 Infragistics, Inc.
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
// </copyright>
//-------------------------------------------------------------------------

using System;
using System.ComponentModel;
using System.Linq;
using System.Collections.Generic;
using IGShowcase.AirplaneSeatingChart.Models;
using Infragistics;
using Infragistics.Controls.Maps;

namespace IGShowcase.AirplaneSeatingChart.ViewModels
{
	public class SeatingMapViewModel : INotifyPropertyChanged
	{
		#region Private Variables

		private readonly PlaneData _planeData;
		private Dictionary<string, SeatViewModel> _seats;
		private readonly SeatClass _allClass;
		private IEnumerable<SeatClass> _classes;
		private SeatClass _selectedClass;
		private IEnumerable<SeatViewModel> _mapElements;
		private SeatViewModel _selectedSeat;
		private SeatViewModel _hoveredSeat;

		private BrushCollection _seatBrushes;
		#endregion Private Variables

		#region Constructors
		/// <summary>
		/// Initializes a new instance of the <see cref="SeatingMapViewModel"/> class.
		/// </summary>
		public SeatingMapViewModel()
		{
			//Load the plane data from the XAML file.
			_allClass = new SeatClass
			           	{
                            SeatCategory = Resources.AppStrings.All,
                            Description = Resources.AppStrings.Seats,
			           		CustomerSatisfaction = new CustomerSatisfactionDataCollection()
			           	};
			_planeData = PlaneData.Load();
		}
		#endregion Constructors

		#region Properties
		/// <summary>
		/// Gets or sets the brushes to use for the map coloring.
		/// </summary>
		public BrushCollection SeatBrushes
		{
			get { return _seatBrushes; }
			set
			{
				if (_seatBrushes == value) return;

				_seatBrushes = value;
				OnPropertyChanged("SeatBrushes");
			}
		}

		/// <summary>
		/// Gets the seat classes. Data source for the classes grid and the customer satisfaction chart.
		/// </summary>
		public IEnumerable<SeatClass> Classes
		{
			get { return _classes; }
			private set
			{
				if (_classes == value) return;

				_classes = value;
				OnPropertyChanged("Classes");
			}
		}

		/// <summary>
		/// Gets or sets the selected seat class.
		/// </summary>
		public SeatClass SelectedClass
		{
			get { return _selectedClass; }
			set
			{
				if (_selectedClass == value) return;

				_selectedClass = value;
				OnSelectedClassChanged();
				OnPropertyChanged("SelectedClass");
			}
		}

		/// <summary>
		/// Gets the data source for the map control's seats layer.
		/// </summary>
		public IEnumerable<SeatViewModel> MapElements
		{
			get { return _mapElements; }
			private set
			{
				if (_mapElements == value) return;

				_mapElements = value;
				OnPropertyChanged("MapElements");
			}
		}

		/// <summary>
		/// Gets or sets the selected seat.
		/// </summary>
		public SeatViewModel SelectedSeat
		{
			get { return _selectedSeat; }
			private set
			{
				if (_selectedSeat == value) return;

				if (value != null)
				{
					value.IsSelected = true;
				}

				SeatViewModel prevSelectedSeat = _selectedSeat;
				_selectedSeat = value;

				if (prevSelectedSeat != null)
				{
					prevSelectedSeat.IsSelected = false;
				}

				OnPropertyChanged("SelectedSeat");
			}
		}

		/// <summary>
		/// Gets or sets the hovered seat.
		/// </summary>
		public SeatViewModel HoveredSeat
		{
			get { return _hoveredSeat; }
			private set
			{
				if (_hoveredSeat == value) return;

				_hoveredSeat = value;
				OnPropertyChanged("HoveredSeat");
			}
		}

		#region Amenities
		/// <summary>
		/// Gets a value indicating whether this instance has food.
		/// </summary>
		/// <value><c>true</c> if this instance has food; otherwise, <c>false</c>.</value>
		public bool HasFood { get { return _planeData.HasFood; } }
		/// <summary>
		/// Gets a value indicating whether this instance has audio.
		/// </summary>
		/// <value><c>true</c> if this instance has audio; otherwise, <c>false</c>.</value>
		public bool HasAudio { get { return _planeData.HasAudio; } }
		/// <summary>
		/// Gets a value indicating whether this instance has video.
		/// </summary>
		/// <value><c>true</c> if this instance has video; otherwise, <c>false</c>.</value>
		public bool HasVideo { get { return _planeData.HasVideo; } }
		/// <summary>
		/// Gets a value indicating whether this instance has power.
		/// </summary>
		/// <value><c>true</c> if this instance has power; otherwise, <c>false</c>.</value>
		public bool HasPower { get { return _planeData.HasPower; } }
		/// <summary>
		/// Gets a value indicating whether this instance has internet.
		/// </summary>
		/// <value>
		/// 	<c>true</c> if this instance has internet; otherwise, <c>false</c>.
		/// </value>
		public bool HasInternet { get { return _planeData.HasInternet; } }
		/// <summary>
		/// Gets a value indicating whether this instance has infant.
		/// </summary>
		/// <value>
		/// 	<c>true</c> if this instance has infant; otherwise, <c>false</c>.
		/// </value>
		public bool HasInfant { get { return _planeData.HasInfant; } }
		#endregion
		#endregion Properties

		#region Public Methods
		/// <summary>
		/// Registers the map elements from the seats layer.
		/// </summary>
		public void RegisterMapElements(IEnumerable<MapElement> elements)
		{
			//Check if the seat brushes are set before calling this method.
			if (_seatBrushes == null || _seatBrushes.Count != 6)
			{
				throw new InvalidOperationException("SeatBrushes property must be set to a collection of 6 brushes before this method is called.");
			}

			//NOTE: Zero out the seats count if set in the data file as those will be counted from the shape file.
			foreach (SeatClass seatClass in _planeData.Classes)
			{
				seatClass.SeatCount = 0;
			}

			_seats = new Dictionary<string, SeatViewModel>(_planeData.Seats.Count);
			foreach (MapElement mapElement in elements)
			{
				string id = mapElement.Name;

				//Get the seat position and class from the shape file.
				string row = Convert.ToString(mapElement.GetProperty("Row"));
				string column = Convert.ToString(mapElement.GetProperty("Column"));
				string seatClassName = Convert.ToString(mapElement.GetProperty("Class"));

				//Lookup the seat information from the XAML data file and create default if not present.
				Seat seat = _planeData.Seats.FirstOrDefault(x => x.Row == row && x.Column == column) ??
							new Seat { Row = row, Column = column, IsOccupied = false, SeatType = SeatType.StandardSeat };
				//Set the seat class from the shape file. Overrides the value set in the data file as it is assumed the
				//shape file holds the plane structure information.
				seat.Class = seatClassName;

				//Count the seats in the classes as we go through the shape file elements.
                SeatClass seatClass = _planeData.Classes.FirstOrDefault(x => x.SeatCategory == seatClassName);
				if (seatClass == null)
				{
					throw new ArgumentException("Seat class not found in data file.");
				}

				++seatClass.SeatCount;

				_seats.Add(id, new SeatViewModel(id, seat, _seatBrushes));
			}
			
			_allClass.SeatCount = elements.Count();

			// NOTE: Prepend the "All" entry and set the classes property after the seats count has been calculated.
			Classes = Enumerable.Repeat(_allClass, 1).Concat(_planeData.Classes).ToArray();
			SelectedClass = _allClass;
			MapElements = _seats.Values;
		}

		/// <summary>
		/// Gets the map element view model based on the specified id.
		/// </summary>
		/// <param name="id">The id.</param>
		/// <returns></returns>
		public SeatViewModel GetMapElementViewModel(string id)
		{
			return _seats[id];
		}

		/// <summary>
		/// Selects the element based on its id.
		/// </summary>
		/// <param name="id">The id.</param>
		public void SelectElement(string id)
		{
			if (SelectedSeat == null || SelectedSeat.Id != id)
			{
				//_seats always contains key Id because the dictionary was created to contain all element names 
				//(see method RegisterMapElements)
				SeatViewModel seat = _seats[id];
				if (!seat.IsDisabled && !seat.IsOccupied)
				{
					SelectedSeat = seat;
					RefreshMapElements();
				}
			}
		}

		/// <summary>
		/// Sets the Hover element based on Id.
		/// </summary>
		/// <param name="id">The id.</param>
		public void HoverElement(string id)
		{
			HoveredSeat = _seats[id];
		}

		/// <summary>
		/// Clears the hover element.
		/// </summary>
		public void UnhoverElement()
		{
			HoveredSeat = null;
		}
		#endregion Public Methods

		#region Private Mehods
		/// <summary>
		/// Called when [selected class changed].
		/// </summary>
		private void OnSelectedClassChanged()
		{
			foreach (SeatViewModel seat in _seats.Values)
			{
                seat.IsDisabled = _selectedClass != _allClass && seat.Class != _selectedClass.SeatCategory;
			}

			RefreshMapElements();
		}

		/// <summary>
		/// Sets the seat layer data source to a new instance of collection of seat VMs to work around 
		/// the map not listening to property changed events on the items in the DataSource collection.
		/// </summary>
		private void RefreshMapElements()
		{
			MapElements = (from kvp in _seats
						   select kvp.Value).ToArray();
		}
		#endregion Private Mehods

		#region INotifyPropertyChanged Members

		public event PropertyChangedEventHandler PropertyChanged;

		private void OnPropertyChanged(string propertyName)
		{
			if (PropertyChanged != null)
			{
				PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
			}
		}

		#endregion
	}
}
